<?php

namespace Ems\portalGestor\Util;

class DateUtil
{
    /**
     * Formato brasileiro de datas (dd/mm/yyyy).
     */
    const FORMATO_BRASILEIRO = "d/m/Y";

    /**
     * Formato brasileiro de datas (dd/mm/yyyy).
     */
    const FORMATO_BRASILEIRO_DATAHORA = "d/m/Y H:i:s";

    public static function getAllDaysBetweenDates(string $dateStart, string $dateEnd): array
    {
        $startObj = self::dateStringToObject($dateStart);
        $endObj = self::dateStringToObject($dateEnd);

        if (self::compare($startObj, $endObj) == 1) {
            return [];
        }

        $tempDate = $startObj;
        $days = [];

        do {
            array_push($days, $tempDate);

            $tempDate = self::nextDate($tempDate);
        } while (self::compare($tempDate, $endObj) <= 0);

        return $days;
    }

    public static function dateStringToObject(string $dateString)
    {
        $splittedDate = explode('/', $dateString);

        $dateObject = self::createDateObject(
            intval($splittedDate[0]),
            intval($splittedDate[1]),
            intval($splittedDate[2])
        );

        return $dateObject;
    }

    public static function dateObjectToString(array $dateObject): string
    {
        $day = str_pad($dateObject["day"], 2, '0', STR_PAD_LEFT);
        $month = str_pad($dateObject["month"], 2, '0', STR_PAD_LEFT);
        $year = $dateObject["year"];

        $dateString = "{$day}/{$month}/{$year}";

        return $dateString;
    }

    public static function createDateObject(int $day, int $month, int $year): array
    {
        $dateObject = array(
            "day" => $day,
            "month" => $month,
            "year" => $year,
        );

        return $dateObject;
    }

    public static function isLeapYear(array $dateObject): bool
    {
        $year = $dateObject["year"];

        $isDivisibleBy4 = $year % 4 == 0;
        $isDivisibleBy100 = $year % 100 == 0;
        $isDivisibleBy400 = $year % 400 == 0;

        // Ano divisível por 4 (exceto anos múltiplos de 100 que não
        //são múltiplos de 400).
        $isLeapYear = $isDivisibleBy4 &&
            !($isDivisibleBy100 &&
                !$isDivisibleBy400);

        return $isLeapYear;
    }

    public static function numberOfDaysInMonth(array $dateObject): int
    {
        $month = $dateObject["month"];
        $isLeapYear = self::isLeapYear($dateObject);

        switch ($month) {
            case 1:
            case 3:
            case 5:
            case 7:
            case 8:
            case 10:
            case 12:
                $daysInMonth = 31;
                break;
            case 4:
            case 6:
            case 9:
            case 11:
                $daysInMonth = 30;
                break;
            case 2:
                $daysInMonth = $isLeapYear ? 29 : 28;
                break;
            default:
                $daysInMonth = 0;
        };

        return $daysInMonth;
    }

    public static function compare(array $date1, array $date2)
    {
        // Compare years
        if ($date1["year"] > $date2["year"]) {
            return 1;
        }

        if ($date1["year"] < $date2["year"]) {
            return -1;
        }

        // Compare months
        if ($date1["month"] > $date2["month"]) {
            return 1;
        }

        if ($date1["month"] < $date2["month"]) {
            return -1;
        }

        // Compare days
        if ($date1["day"] > $date2["day"]) {
            return 1;
        }

        if ($date1["day"] < $date2["day"]) {
            return -1;
        }

        // Same date
        return 0;
    }

    public static function nextDate(array $dateObject)
    {
        $day = $dateObject["day"];
        $month = $dateObject["month"];
        $year = $dateObject["year"];

        if ($day == self::numberOfDaysInMonth($dateObject)) {
            $nextDay = 1;

            if ($month == 12) {
                $nextMonth = 1;
                $nextYear = $year + 1;
            } else {
                $nextMonth = $month + 1;
                $nextYear = $year;
            }
        } else {
            $nextDay = $day + 1;
            $nextMonth = $month;
            $nextYear = $year;
        }

        $nextDateObject = self::createDateObject($nextDay, $nextMonth, $nextYear);

        return $nextDateObject;
    }

    /**
     * Retorna um objeto do tipo DateTime de uma string passada por parametro. Por padrao, o formato de
     * data utilizado é o formato brasileiro (dd/mm/yyyy).
     *
     * @static
     * @param $data
     * @param string $formato
     * @return \DateTime
     */
    public static function getDataDeString(
        $data,
        $formato = self::FORMATO_BRASILEIRO,
        $truncate = false
    ) {
        try {
            $data = \DateTime::createFromFormat($formato, $data);

            if ($truncate) {
                $data = self::truncateData($data);
            }

            return $data;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function getWeekDay(\DateTime $data) {
        return $data->format('w');
    }

    /**
     * Retorna a mesma data passada por parametro truncada (sem data, hora e minuto).
     *
     * @static
     * @param DateTime $data
     * @return DateTime
     */
    public static function truncateData(\DateTime $data) {
        $strData = $data->format(self::FORMATO_BRASILEIRO);
        $strData .= " 00:00:00";
        return \DateTime::createFromFormat(self::FORMATO_BRASILEIRO_DATAHORA, $strData);
    }
}
